function [Dt,Sigmag,covmtx]=GMM_multiP_var_SARX_het(parm,Y,L,Q,P,Wg,W,N,k,longv)
% Computes GMM variance assuming heteroskedastic errors
% longv==1: uses general sandwich formula
% longv==0: uses optimal GMM variance formula

rhohat = parm(1);
betahat = parm(2:k+1);
X = L(:,2:end);
q = size(Q,2);
nP = size(P,3);
S = speye(N)-rhohat*W; 
G = W/S;

% Construct D matrix
Dt = zeros(k+1,q+nP);   % D transpose
Dt(1,nP+1:q+nP) = (G*X*betahat)'*Q;
Dt(2:k+1,nP+1:q+nP) = X'*Q;

% Construct variance of moments: Sigmag
ehatN = Y-L*parm; % N by 1
e2hatN = ehatN.^2;
Delta1 = zeros(N*N,nP); 
Delta2 = zeros(N*N,nP);
for j = 1:nP
    Pt = P(:,:,j)';
    Ps = P(:,:,j)+Pt;
    SimgaPt = (e2hatN.*P(:,:,j))';
    SigmaPs = e2hatN.*Ps;
    Delta1(:,j) = SimgaPt(:); 
    Delta2(:,j) = SigmaPs(:);
    Dt(1,j) = sum(sum(e2hatN'.*Ps.*G));
end
Dt = Dt/N;
D = Dt';     

V = zeros(q+nP,q+nP);
V(1:nP,1:nP) = Delta1'*Delta2;
V(nP+1:q+nP,nP+1:q+nP) = Q'*(e2hatN.*Q);
Sigmag = V/N;

if longv==1
    DWg = Dt*Wg;
    WgD = Wg*D;
    DWgD = DWg*D;
    mid = DWg*Sigmag*WgD;
    covmtx = pinv(DWgD)*mid*pinv(DWgD)/N;
elseif longv==0
    tmp = Dt*pinv(Sigmag)*D;
    covmtx = pinv(tmp)/N;
end

end