clear all;
close all;
load('.\data_application.mat');
%% 
sample = 1;   % =1(or 2) first (second) subsample
est_option = 5.2;   % change the value of est_option to run different estimations  
            % BMM assuming homoskedasticity=3; BMM assuming heteroskedasticity=3.1
            % GMM/OGMM/BGMM assuming homoskedasticity=4, 4.1, 4.2, respectively
            % GMM/OGMM/BGMM assuming heteroskedasticity=5, 5.1, 5.2, respectively
flag_est_delta = 1; % set to zero if want to suppress estimating delta-dominance
eps_W = 0.1;  % threshold value for robust W

% %========================================================================
if sample==1	  % first subsample (1998-2006)
    W_raw = W_2002;
    Price = Price_W2002(:,1:10);  % price level
    Wstring = 'W2002';
elseif sample==2  % second subsample (2007-2015)
    W_raw = W_2007;
    Price = Price_W2007(:,10:19);
    Wstring = 'W2007';  
end

if min(min(W_raw))<0
    error('W contains negative elements!')
end
%  W_raw(W_raw<0)=0;     % set negative entries to zeros
W_raw = normw(W_raw);  % NOTE: Wii is NOT zero!

% construct robust W
W_rbst_tmp = rbstw(W_raw,eps_W);  % NOTE: created new zero rows!
rowsum = sum(W_rbst_tmp,2);
idx_zerorow = find(~rowsum);   % W_2002: 26 zero rows; W_2007: 33 zero rows
idx_nzrow = find(rowsum);  % find indices of nonzero elements
W_rbst_nzrow = W_rbst_tmp(idx_nzrow,idx_nzrow); % drop zero rows
W_rbst_nzrow = normw(W_rbst_nzrow);      
Price_nzrow = Price(idx_nzrow,:);  % drop zero rows
% NOTE: created new zero rows!
rowsum_2nd = sum(W_rbst_nzrow,2);
idx_zerorow_2nd = find(~rowsum_2nd);   % drop the newly created zero rows
idx_nzrow_2nd = find(rowsum_2nd); 
W_rbst_nzrow_2nd = W_rbst_nzrow(idx_nzrow_2nd,idx_nzrow_2nd);
W_rbst_nzrow_2nd = normw(W_rbst_nzrow_2nd);
Price_nzrow_2nd = Price_nzrow(idx_nzrow_2nd,:);  % drop the newly created zero rows 
if min(sum(W_rbst_nzrow_2nd,2))==0   % not need for the threshold values considered
    error('zero row sums after 2nd round!');
else
    W = W_rbst_nzrow_2nd;
    pNT = log(Price_nzrow_2nd);
end
W2 = W*W;

if min(sum(W,2))==0
    warning('W has zero row sums!');  
end
% %========================================================================
% estimate delta-dominance
if flag_est_delta~=0
    M = 5;   % report top five
    delta_W_raw = est_delta(sum(W_raw),M);  % call est_delta function
    fprintf('   delta-dominance of raw W \n')
    disp(delta_W_raw.deltaM)

    delta_W = est_delta(sum(W),M);
    fprintf('   delta-dominance of robust W\n')
    disp(delta_W.deltaM)
    fprintf('   raw W nz= %i, %3.2f \n',nnz(W_raw), nnz(W_raw)/numel(W_raw)*100)
    fprintf('   robust W nz= %i, %3.2f \n',nnz(W), nnz(W)/numel(W)*100)
end
N = size(W,1);
dpNT = (pNT(:,2:end) - pNT(:,1:end-1))*100;  % in percentages
Y = mean(dpNT,2);
Ystar = W*Y;
X = ones(N,1);
k = size(X,2);
L = [Ystar,X];  % regressors
outtab = [];
%% start estimating the model
if round(est_option)==3     % BMM estimation   
    Mb = 1000; 
    if est_option==3
        info.parm_lb = [0;-Mb*ones(k,1);0];  
        info.parm_ub = [1;Mb*ones(k+1,1)];
        info.parm0 = [0;zeros(k,1);1];
        results_BMM = BMM_SARX(Y,X,Ystar,W,0,info);
        est_option_name = 'BMM homoskedascity';
    elseif est_option==3.1
        info.parm_lb = [0;-Mb*ones(k,1)];  
        info.parm_ub = [1;Mb*ones(k,1)];
        info.parm0 = [0;zeros(k,1)];
        results_BMM = BMM_SARX(Y,X,Ystar,W,1,info);
        est_option_name = 'BMM heteroskedascity';        
    end
    rhohat = results_BMM.rho;
    betahat = results_BMM.beta;
    se_rhohat = results_BMM.se_rho;
    se_betahat = results_BMM.se_beta;
    parmhat = [rhohat; betahat];
    se = [se_rhohat;se_betahat];
    out = [parmhat';se']; 
% %========================================================================            
elseif est_option>=4          % GMM estimation
    clear options P Wg;         
    Q = X;     % intercept only; this is a pure SAR model
    q = size(Q,2);
    if est_option<5  % assuming homoskedascity
        P(:,:,1) = W - (1/N)*trace(W)*eye(N);
        P(:,:,2) = W2 - (1/N)*trace(W2)*eye(N);
    elseif est_option>=5  % assuming heteroskedascity
        P(:,:,1) = W-diag(diag(W));
        P(:,:,2) = W2-diag(diag(W2));
    end
    nP = size(P,3);
    Wg = eye(q+nP);       
    options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
%     options.Display = 'iter';
%     options.Display = 'iter-detailed'; 
    options.MaxFunEvals = 1e6;
    options.MaxIter = 1e6;
    options.TolFun = 1e-6;  
    parm0 = [0.5; ones(k,1)];
    Mb = 10000;                  
    parm_lb = [-1;-Mb*ones(k,1)];    
    parm_ub = [1;Mb*ones(k,1)];    
    [parmhat,~,~,exitflag] = lsqnonlin(@GMM_multiP_moment_SARX,parm0,parm_lb,parm_ub,options,Y,L,Q,P,Wg,N,1,0);
    if exitflag~=1 && exitflag~=3
        error('convergence error');
    end    
    if abs(parmhat(1))>1
        error('rhohat exceeds bounds!')
    end
    if est_option<5
        [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_iid(parmhat,Y,L,Q,P,Wg,W,N,k,1);
         est_option_name = 'GMM homoskedascity';
    else
        [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_het(parmhat,Y,L,Q,P,Wg,W,N,k,1);
         est_option_name = 'GMM heteroskedascity';
    end     
    if est_option==4.1 || est_option==5.1     % OGMM estimation    
        Wg = pinv(Sigmag);
        [parmhat,~,~,exitflag] = lsqnonlin(@GMM_multiP_moment_SARX,parmhat,parm_lb,parm_ub,options,Y,L,Q,P,Wg,N,1,0);
        if exitflag~=1 && exitflag~=3
            error('convergence error');
        end
        if est_option==4.1
            [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_iid(parmhat,Y,L,Q,P,Wg,W,N,k,1);    
             est_option_name = 'OGMM homoskedascity';
        elseif est_option==5.1
            [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_het(parmhat,Y,L,Q,P,Wg,W,N,k,1);
             est_option_name = 'OGMM heteroskedascity';
        end
    elseif est_option==4.2 || est_option==5.2    % BGMM estimation
        parmini = parmhat;   % use GMM estimates as initial values
        Shat = eye(N)-parmini(1)*W;
        Ghat = W/Shat;
        if est_option==4.2
            Phat = Ghat-trace(Ghat)/N*eye(N);
        elseif est_option==5.2
            Phat = Ghat - diag(diag(Ghat));
        end
        Qhat = X;
        q = size(Qhat,2);
        Wg = eye(q+1);   
        [parmhat,~,~,exitflag] = lsqnonlin(@GMM_multiP_moment_SARX,parm0,parm_lb,parm_ub,options,Y,L,Qhat,Phat,Wg,N,1,0);
        if exitflag~=1 && exitflag~=3
            error('convergence error');
        end    
        if abs(parmhat(1))>1
            error('rhohat exceeds bounds!')
        end
        if est_option==4.2
            [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_iid(parmhat,Y,L,Qhat,Phat,Wg,W,N,k,1);
        elseif est_option==5.2
            [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_het(parmhat,Y,L,Qhat,Phat,Wg,W,N,k,1);
        end
        Wg = pinv(Sigmag);
        [parmhat,~,~,exitflag] = lsqnonlin(@GMM_multiP_moment_SARX,parmhat,parm_lb,parm_ub,options,Y,L,Qhat,Phat,Wg,N,1,0);
        if exitflag<1
            warning('convergence error');
            exitflag
        end
        if est_option==4.2
            [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_iid(parmhat,Y,L,Qhat,Phat,Wg,W,N,k,1);
            est_option_name = 'BGMM homoskedascity';
        elseif est_option==5.2
            [Dt,Sigmag,covmtx] = GMM_multiP_var_SARX_het(parmhat,Y,L,Qhat,Phat,Wg,W,N,k,1);
            est_option_name = 'BGMM heteroskedascity';
        end
    end
    rhohat = parmhat(1);
    if abs(rhohat)>1
        warning('rhohat exceeds bounds!');
    end
    eps_bound = 1e-3;
    if abs(rhohat - (-1))<eps_bound
       warning('lower boundary solution!')
       rhohat = -1+eps_bound;
    elseif abs(rhohat-1)<eps_bound
       warning('upper boundary solution!')   
       rhohat = 1-eps_bound;
    end                   
    betahat = parmhat(2:end);
    se = sqrt(diag(covmtx));
    se_rhohat = se(1);
    se_betahat = se(2:end);
    out = [parmhat';se'];
end  

% print out results
outvec = out(:);   % [rho; intercept]
disp('   ---------------')
disp(['    ', est_option_name])
fprintf(repmat('    %2.3f\n   (%2.3f)\n',1,length(outvec)/2), outvec)

if est_option==3 || round(est_option)==4
    % estimate error variance, sige2hat
    resid = Y-L*parmhat;
    sige2hat = sum(resid.^2)/(N-size(L,2)-1);        
    fprintf('    sige2hat=%2.3f\n', sige2hat)
end

% Compute R2
Sy = eye(N) - rhohat*W;
invSy = inv(Sy);
if est_option==3 || round(est_option)==4   % homoskedasticity
    R2 = 1 - N/trace(invSy*invSy');
    fprintf('    R2=%2.3f\n', R2)
end


    

